#include "Input.h"

namespace mdd{
    int Input::addConnection(std::shared_ptr<IOutput> output)
    {
        _output = output;
        return 1;
    }
    int Input::removeConnection(std::shared_ptr<IOutput> output)
    {
        _output = nullptr;
        return 0;
    }
    Input::Input(IModule* parent, const std::string& name, int appendix, const std::vector<double>& default_value) {
        _parent = parent;
        _name = name;
        _value = default_value;
        _appendix = appendix;
        _optimizable = false;

        type = "input";
        key = "Input";
    }

    std::string Input::getType() {
        return  type;
    }

    std::string Input::getGeneratorKey() {
        return  key;
    }
 
    std::string Input::setName(const std::string& name){
        _name = name;
        return getID();
    }

    std::string Input::getName() {
        return  _name;
    }
    
    std::string Input::setAppendix(int appendix){
        _appendix = appendix;
        return  getID();
    }

    int Input::getAppendix() {
        return _appendix;
    }

    std::string Input::getID() {
        return _name + std::to_string(_appendix);
    }

    std::string Input::getParentID() {
        return _parent->getID();
    }

    state Input::getState() {
        if (_output == nullptr) {
            return state::UNCHANGED;
        }
        else {
            return _output->getState();
        }
    }

    void Input::resetState() {

    }

    const std::vector<double>& Input::getValue() {
        if(_output == nullptr){
            return _value;
        } else{
            return _output->getValue();
        }
    }

    std::shared_ptr<IOutput> Input::getConnection() { return _output; }

    std::vector<double>& Input::setValue(){
        return _value;
    }

    state Input::setValue(const std::vector<double>& val) {
        if (val != _value)
        {
            _value = val;
            return state::CHANGED;
        }
        else {
            return state::UNCHANGED;
        }
    }

    bool Input::connect(std::shared_ptr<IOutput> output){
        _output = output;
        _output->addConnection(std::make_shared<Input>((*this)));
        return true;
    }

    void Input::disconnect() {
        if (_output != nullptr)
        {
            _output->removeConnection(std::make_shared<Input>((*this)));
        }
    }

    bool Input::isOptimizable() {
        return _optimizable;
    }

    void Input::setOptimizability(bool state) {
        _optimizable = state;
    }
    const limits& Input::getLimits()
    {
        return _limit;
    }
    limits& Input::setLimits()
    {
        return _limit;
    }

    bool Input::configure(const std::string& config) 
    {
        json jconfig = json::parse(config);
        if (jconfig.contains("name"))
        {
            _name = jconfig["name"].get < std::string >();
        }
        if (jconfig.contains("appendix"))
        {
            _appendix = jconfig["appendix"].get < int >();
        }
        if (jconfig.contains("optimizable"))
        {
            _optimizable = jconfig["optimizable"].get < bool >();
        }
        if (jconfig.contains("value"))
        {
            _value = jconfig["value"].get <std::vector<double> >();
        }
        return true;
    }

    std::string Input::getConfiguration() 
    {
        json jconfig;
        jconfig["name"] = _name;
        jconfig["appendix"] = _appendix;
        jconfig["value"] = _value;
        jconfig["prefix"].push_back(_parent->getID());
        jconfig["optimizable"] = _optimizable;
        return jconfig.dump();
    }

    void Input::load(const json& j) 
    {
        configure(j.dump());
        return;
    }

    json Input::dump() 
    {
        json jdump = json::parse(getConfiguration());
        return jdump;
    }
}