#include "OptimizerBase.h"
#include <iostream>

namespace mdd{
	bool OptimizerBase::connect(std::shared_ptr<IModule> module) {
		_module = module;
		return true;
	}

	OptimizerBase::opt_state OptimizerBase::updateOutputs()
	{
		opt_state ret;
		ret.module_state = _module->update();
		if (ret.module_state != state::STATE_ERROR)
		{
			for (size_t i = 0; i < _output_vals.size(); ++i)
			{
				_output_vals[i] = _outputs[i]->getValue()[0];
				//std::cout << "get: " << i << ": " << _output_vals[i] << std::endl;
			}
			ret.opt_value = _func_expr.value();
			//std::cout << "get: " << "opt" << ": " << ret.opt_value << std::endl;
		}
		
		return ret;
	}

	OptimizerBase::OptimizerBase(const std::string& base_config) {
		_base_config = base_config;
	}

	void OptimizerBase::updateLayout()
	{
		_inputs = _module->getOptimizableInputs();
		_outputs = _module->getOptimizableOutputs();
		_output_vals.clear();
		if (_inputs.empty())
		{
			std::cout << "ERROR: No optimizable inputs detected!" << std::endl;
		}
		if (_outputs.empty())
		{
			std::cout << "ERROR: No optimizable outputs detected!" << std::endl;
		}
		for (auto& out : _outputs)
		{
			_output_vals.push_back(out->getValue()[0]);
		}
	}
	
	bool OptimizerBase::setEvaluation(std::string func)
	{
		updateLayout();
		exprtk::symbol_table<double> symbol_table;
		for (size_t i = 0; i < _output_vals.size(); ++i)
		{
			symbol_table.add_variable("out" + std::to_string(i), _output_vals[i]);
		}
		symbol_table.add_constants();
	
		_func_expr.register_symbol_table(symbol_table);
	
		exprtk::parser<double> parser;
		return parser.compile(func, _func_expr);
	}
	
	bool OptimizerBase::configure(const std::string& config) {
		json config_parsed = json::parse(config);
		for (size_t i = 0; i < config_parsed.size(); i++)
		{
			if (config_parsed[i].contains("name"))
			{
				if (config_parsed[i]["name"].get<std::string>() == "operation")
				{
					const std::string op = config_parsed[i]["value"].get<std::string>();
					
				}
			}
		}
		return false;
	}
	std::string OptimizerBase::getConfiguration() {
		return _base_config;
	}
	//virtual std::string getGeneratorID() = 0;
	void OptimizerBase::load(const json& j) {
		if (j.contains("configure"))
		{
			configure(j["configure"].get<std::string>());
		}
	}
	json OptimizerBase::dump() {
		json ret;

		ret["configure"] = _base_config;

		return ret;
	}
}