#include "Output.h"
namespace mdd {
    int Output::addConnection(std::shared_ptr<IInput> input)
    {
        _connections.push_back(input);
        return _connections.size();
    }
    int Output::removeConnection(std::shared_ptr<IInput> input)
    {
        for (auto it = _connections.begin(); it != _connections.end(); ++it) {
            if ((*it) == input)
            {
                _connections.erase(it);
            }
        }
        return _connections.size();
    }
    Output::Output(IModule* parent, const std::string& name, int appendix, const std::vector<double>& initial) :
            _state(state::UNCHANGED),
            _parent(parent)
    {
        _appendix = appendix;
        _name = name;
        _value = initial;
        _optimizable = false;

        type = "output";
        key = "Output";
    }

    const std::vector<double>& Output::getValue() { return _value; }

    std::vector<double>& Output::setValue() {
        _state = state::CHANGED;
        return _value;
    }
    state Output::setValue(const std::vector<double>& val) {
        if (val != _value)
        {
            _value = val;
            _state = state::CHANGED;
        }
        else {
            _state = state::UNCHANGED;
        }
        return _state;
    }
    bool Output::isOptimizable() {
        return _optimizable;
    }

    void Output::setOptimizability(bool state) {
        _optimizable = state;
    }

    state Output::getState() {
        return _state;
    }
    
    void Output::resetState() {
        _state = state::UNCHANGED;
    }

    std::string Output::getType() {
        return  type;
    }

    std::string Output::getGeneratorKey() {
        return  key;
    }

    std::string Output::setName(const std::string& name) {
        _name = name;
        return getID();
    }

    std::string Output::getName() {
        return  _name;
    }

    std::string Output::setAppendix(int appendix) {
        _appendix = appendix;
        return  getID();
    }

    int Output::getAppendix() {
        return _appendix;
    }

    std::string Output::getID() {
        return _name + std::to_string(_appendix);
    }

    std::string Output::getParentID() {
        return _parent->getID();
    }

    bool Output::connect(std::shared_ptr<IInput> input)
    {
        addConnection(input);
        input->addConnection(std::make_shared<Output>((*this)));
        return true;
    }
    std::vector<std::shared_ptr<IInput>> Output::getConnections()
    {
        return _connections;
    }
    void Output::disconnect() {
        for (size_t i = 0; i < _connections.size(); i++)
        {
            if (_connections[i] != nullptr)
            {
                _connections[i]->removeConnection(std::make_shared<Output>((*this)));
            }
        }
        _connections.clear();
    }

    bool Output::configure(const std::string& config)
    {
        json jconfig = json::parse(config);
        if (jconfig.contains("name"))
        {
            _name = jconfig["name"].get < std::string >();
        }
        if (jconfig.contains("appendix"))
        {
            _appendix = jconfig["appendix"].get < int >();
        }
        if (jconfig.contains("optimizable"))
        {
            _optimizable = jconfig["optimizable"].get < bool >();
        }
        if (jconfig.contains("value"))
        {
            _value = jconfig["value"].get <std::vector<double> >();
        }
        return true;
    }

    std::string Output::getConfiguration()
    {
        json jconfig;
        jconfig["name"] = _name;
        jconfig["appendix"] = _appendix;
        jconfig["value"] = _value;
        jconfig["prefix"].push_back(_parent->getID());
        jconfig["optimizable"] = _optimizable;
        return jconfig.dump();
    }

    void Output::load(const json& j)
    {
        configure(j.dump());
        return;
    }

    json Output::dump()
    {
        json jdump = json::parse(getConfiguration());
        return jdump;
    }
}