#include "Parameter.h"

namespace mdd {
    Parameter::Parameter(IModule* parent)
        : ModuleBase(R"JSON(
        [{
            "name":"appendix",
            "value":0
        }])JSON")
    {
        _parent = parent;
        std::vector<double> default_val = { 1 };
        inputs.push_back(std::make_shared<Input>(parent, "Value", 0, default_val));
        outputs.push_back(std::make_shared<Output>(parent, "Value", 0, default_val));
        
        key = "Parameter";
        setName(key);
    }

    state Parameter::update() {
        return getOutput(0)->setValue(getInput(0)->getValue());
    }

    std::string Parameter::setName(const std::string& name) {
        inputs[0]->setName(name);
        outputs[0]->setName(name);
        return ModuleBase::setName(name);
    }

    std::string Parameter::setAppendix(int appendix)
    {
        inputs[0]->setAppendix(appendix);
        outputs[0]->setAppendix(appendix);
        return ModuleBase::setAppendix(appendix);
    }
    bool Parameter::configure(const std::string& config)
    {
        json config_parsed = json::parse(config);
        for (size_t i = 0; i < config_parsed.size(); i++)
        {
            if (config_parsed[i].contains("name"))
            {
                if (config_parsed[i]["name"].get<std::string>() == "appendix")
                {
                    setAppendix(config_parsed[i]["value"].get<int>());
                }
            }
        }

        return false;
    }

    std::string Parameter::getConfiguration() {
        json ret = json::parse(ModuleBase::getConfiguration());
        for (size_t i = 0; i < ret.size(); i++)
        {
            if (ret[i].contains("name"))
            {
                if (ret[i]["name"].get<std::string>() == "appendix")
                {
                    ret[i]["value"] = getAppendix();
                }
            }
        }

        return ret.dump();
    }
}